package com.greenorange.promotion.model.dto.advancementApply;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;
import java.math.BigDecimal;

import java.io.Serial;
import java.io.Serializable;

/**
 * 晋升申请更新请求体
 */
@Data
@Schema(description = "晋升申请更新请求体", requiredProperties = {
                "id",
                "name",
                "phone",
                "verificationCode",
                "resume",
                "reviewStatus",
})
public class AdvancementApplyUpdateRequest implements Serializable {

    /**
     * 晋升申请id
     */
    @Min(value = 1L, message = "晋升申请id ID不能小于1")
    @Schema(description = "晋升申请id", example = "张三")
    private Long id;

    /**
     * 申请人姓名
     */
    @NotBlank(message = "申请人姓名不能为空")
    @Schema(description = "申请人姓名", example = "")
    private String name;

    /**
     * 申请人手机号
     */
    @NotBlank(message = "申请人手机号不能为空")
    @Schema(description = "申请人手机号", example = "15888610253")
    private String phone;

    /**
     * 验证码
     */
    @NotBlank(message = "验证码不能为空")
    @Schema(description = "验证码", example = "666999")
    private String verificationCode;

    /**
     * 简历(view值)
     */
    @NotBlank(message = "简历(view值)不能为空")
    @Schema(description = "简历(view值)", example = "3DFK2K3J")
    private String resume;

    /**
     * 审核状态
     */
    @NotBlank(message = "审核状态不能为空")
    @Schema(description = "审核状态", example = "待审核")
    private String reviewStatus;


    @Serial
    private static final long serialVersionUID = 1L;
}