package com.greenorange.promotion.model.dto.course;

import com.greenorange.promotion.annotation.EnumValue;
import com.greenorange.promotion.model.enums.CourseTypeEnum;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotNull;
import jdk.jfr.BooleanFlag;
import lombok.Data;
import java.math.BigDecimal;

import java.io.Serial;
import java.io.Serializable;
import com.greenorange.promotion.common.PageRequest;

/**
 * 课程查询请求体，继承自分页请求 PageRequest
 */
@Data
@Schema(description = "课程查询请求体", requiredProperties = {"current", "pageSize"})
public class CourseQueryRequest extends PageRequest implements Serializable {

    /**
     * 课程名称
     */
    @Schema(description = "课程名称", example = "数据分析工程师训练营")
    private String name;

    /**
     * 课程类别[考公,财经]
     */
    @EnumValue(enumClass = CourseTypeEnum.class)
    @Schema(description = "课程类别[考公,财经]", example = "自媒体")
    private String type;


    /**
     * 是否上架(true:上架，false:下架)
     */
    @Schema(description = "是否上架(true:上架，false:下架)", example = "true")
    private Boolean isShelves;


    @Serial
    private static final long serialVersionUID = 1L;
}

