package com.greenorange.promotion.model.dto.userInfo;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Size;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;

/**
 * 用户添加请求体
 */
@Data
@Schema(description = "用户添加请求体", requiredProperties = {"nickName", "userAvatar", "phoneNumber",
        "userAccount", "userPassword"})
public class UserInfoAddRequest implements Serializable {

    /**
     * 用户昵称
     */
    @NotBlank(message = "用户昵称不能为空")
    @Schema(description = "用户昵称", example = "chenxinzhi")
    private String nickName;

    /**
     * 用户头像URL
     */
    @NotBlank(message = "用户头像URL不能为空")
    @Schema(description = "用户头像URL", example = "3E8U2AM8")
    private String userAvatar;

    /**
     * 手机号
     */
    @NotBlank(message = "手机号不能为空")
    @Schema(description = "手机号", example = "15888610253")
    private String phoneNumber;

    /**
     * 账号
     */
    @NotBlank(message = "账号不能为空")
    @Size(min = 6, max = 11, message = "账号长度在 6 到 11 个字符")
    @Schema(description = "账号", example = "qingcheng")
    private String userAccount;

    /**
     * 密码
     */
    @NotBlank(message = "密码不能为空")
    @Size(min = 6, max = 10, message = "密码长度在 6 到 10 个字符")
    @Schema(description = "密码（建议加密存储）", example = "qingcheng")
    private String userPassword;


    @Serial
    private static final long serialVersionUID = 1L;
}
