package com.greenorange.promotion.model.dto.userInfo;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Size;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;

/**
 * 小程序用户密码登录请求体
 */
@Data
@Schema(description = "小程序用户密码登录请求体", requiredProperties = {"phoneNumber", "userPassword"})
public class UserInfoMiniPasswordLoginRequest implements Serializable {

    /**
     * 账号
     */
    @NotBlank(message = "手机号不能为空")
    @Schema(description = "手机号", example = "15888610253")
    private String phoneNumber;

    /**
     * 密码
     */
    @NotBlank(message = "密码不能为空")
    @Size(min = 6, max = 10, message = "密码长度在 6 到 10 个字符")
    @Schema(description = "密码（建议加密存储）", example = "qingcheng")
    private String userPassword;

    @Serial
    private static final long serialVersionUID = 1L;

}
