package com.greenorange.promotion.model.enums;

import com.greenorange.promotion.annotation.BaseEnum;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 课程类型枚举
 */
@Getter
public enum CourseTypeEnum implements BaseEnum {

    KAOGONG("考公"),
    CAIJING("财经");

    private final String value;

    CourseTypeEnum(String value) {
        this.value = value;
    }

    /**
     * BaseEnum 要求的方法：返回枚举对应的校验值
     */
    @Override
    public String getValue() {
        return this.value;
    }

    /**
     * 获取所有枚举值列表
     */
    public static List<String> getValues() {
        return Arrays.stream(values())
                     .map(CourseTypeEnum::getValue)
                     .collect(Collectors.toList());
    }

    /**
     * 根据值获取对应的枚举对象
     */
    public static CourseTypeEnum getEnumByValue(String value) {
        if (StringUtils.isBlank(value)) {
            return null;
        }
        for (CourseTypeEnum type : CourseTypeEnum.values()) {
            if (type.value.equals(value)) {
                return type;
            }
        }
        return null;
    }
}
